@@ -1,5 +1,5 @@
 __all__ = ['dcmreaddata', 'datareader', 'datawriter', 'rawN', "datasets"]
-__version__ = "1.3.31"
+__version__ = "1.18.0"
 # from pycut import Model, ImageGraphCut
 # from seed_editor_qt import QTSeedEditor
 from .dcmreaddata import DicomReader
@@ -9,4 +9,5 @@ from .datareader import DataReader
 from .datareader import read
 from .datawriter import write
 from .datasets import download
-
+from .fsbrowser import remove_if_exists
+from imma import image_manipulation
