@@ -10,6 +10,7 @@ $ dcmreaddata -d sample_data -o head.mat
 """
 
 import logging
+logger = logging.getLogger(__name__)
 import os
 import re
 import sys
@@ -17,16 +18,19 @@ import traceback
 from optparse import OptionParser
 
 try:
-    import dicom as pydicom
-except Exception:
-# except ModuleNotFoundError:
     import pydicom
+except ImportError:
+    import warnings
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore")
+        import dicom as pydicom
+    logger.debug("dicom imported - it would be better use pydicom")
+# except ModuleNotFoundError:
 
 import numpy as np
 from scipy.io import savemat
 import os.path as op
 
-logger = logging.getLogger(__name__)
 from . import misc
 from . import dcmtools
 
@@ -66,8 +70,9 @@ def is_dicom_dir(datapath):
             retval = True
         # except pydicom.errors.InvalidDicomError:
         #     logger.debug("Invalid Dicom while reading file " + str(f))
-        except Exception:
+        except Exception as e:
             logger.warning("Unable to read dicom file " + str(f))
+            logger.warning(e)
             # import traceback
             # traceback.print_exc()
 
@@ -97,15 +102,22 @@ def decode_overlay_slice(data, i_overlay):
     # TODO neni tady ta jednička blbě?
     for i in range(1, len(overlay_raw)):
         for k in range(0, n_bits):
-            # Python2 returns str, Python3 returns int. (Could also by caused by slight difference in dicom lib version number)
-            byte_as_int = ord(overlay_raw[i]) if type(overlay_raw[i]) == type(str("")) else overlay_raw[i]
+            # NOTE: Python2 returns str, Python3 returns int.
+            #       Could be by caused by slight difference in dicom lib version number.
+            one_byte = overlay_raw[i]
+            if sys.version_info.major == 2:
+                byte_as_int = ord(one_byte)
+            else:
+                byte_as_int = one_byte
+            # byte_as_int = ord(overlay_raw[i]) if type(overlay_raw[i]) == type(str("")) else overlay_raw[i]
             decoded_linear[i * n_bits + k] = (byte_as_int >> k) & 0b1
 
     # verlay = np.array(pol)
     overlay_slice = np.reshape(decoded_linear, [rows, cols])
     return overlay_slice
 
-class DicomReader():
+
+class DicomReader:
     """
     Example:
 
@@ -137,6 +149,8 @@ class DicomReader():
                 get_series_number_callback = get_series_number_qt
             else:
                 get_series_number_callback = get_series_number_console
+        elif get_series_number_callback == "guess for liver":
+            get_series_number_callback = get_series_number_by_guess_for_liver
 
         self.get_series_number_callback = get_series_number_callback
         self.__check_series_number()
@@ -261,28 +275,21 @@ class DicomReader():
         if stop is None:
             stop = len(dcmlist)
 
-        printRescaleWarning = False
+        # printRescaleWarning = False
         for i in xrange(start, stop, step):
             onefile = dcmlist[i]
             data = self._read_file(onefile)
-            new_data2d, original_dtype = dcmtools.get_pixel_array_from_pdcm(data)
+            new_data2d = data.pixel_array
+            # new_data2d, slope, inter = dcmtools.get_pixel_array_from_pdcm(data)
             # mport pdb; pdb.set_trace()
 
             if len(data3d) == 0:
                 shp2 = new_data2d.shape
                 data3d = np.zeros([len(dcmlist), shp2[0], shp2[1]],
-                                  dtype=np.int16)
-                # mx = np.max(data2d)
-                # mn = np.min(data2d)
-                # if (raw_max is None) or (raw_max < mx):
-                #     raw_max = mx
-                # if (raw_min is None) or (raw_min < mx):
-                #     raw_min = mn
+                                  dtype=new_data2d.dtype)
+                slope, inter = dcmtools.get_slope_and_intercept_from_pdcm(data)
 
 
-            if original_dtype == np.uint16 and data3d.dtype == np.int16:
-                data3d = data3d.astype(np.int32)
-                # or just force set slope=0.5, inter = 0
 
             # first readed slide is at the end
 
@@ -299,16 +306,22 @@ class DicomReader():
             logger.debug("Data size: " + str(data3d.nbytes)
                          + ', shape: ' + str(shp2) + 'x' + str(len(dcmlist))
                          + ' file ' + onefile)
-        if printRescaleWarning:
-            print("Automatic Rescale with slope 0.5")
-            logger.warning("Automatic Rescale with slope 0.5")
+        data3d = misc.use_economic_dtype(data3d, slope=slope, inter=inter)
+        # if original_dtype == np.uint16 and data3d.dtype == np.int16:
+        #     data3d = data3d.astype(np.int32)
+            # or just force set slope=0.5, inter = 0
+        # new_data2d = rescale_pixel_array(data2d, slope, inter)
+        # if printRescaleWarning:
+        #     print("Automatic Rescale with slope 0.5")
+        #     logger.warning("Automatic Rescale with slope 0.5")
+        # data3d = dcmtools.rescale_pixel_array(data3d, slope=slope, inter=inter)
 
         return data3d
 
-    def get_metaData(self, ifile=0):
+    def get_metaData(self):
         dcmlist = self.files_in_serie
         # self.dicomdirectory.get_metadata_new(series_number=self.series_number)
-        return self.dicomdirectory.get_metaData(dcmlist=dcmlist, series_number=self.series_number, ifile=ifile)
+        return self.dicomdirectory.get_metaData(dcmlist=dcmlist, series_number=self.series_number)
 
     def dcmdirstats(self):
         return self.dicomdirectory.get_stats_of_series_in_dir()
@@ -329,6 +342,14 @@ def get_one_serie_info(series_info, serie_number):
             'Tag Modality, SeriesDescription or ImageComment not found in dcminfo'
         )
         pass
+    try:
+        strl = strl + ", " \
+               + str(series_info[serie_number]['StudyDate'])
+    except Exception:
+        logger.debug(
+            'Tag StudyDate not found in dcminfo'
+        )
+        pass
     strl = strl + ')'
     return strl
 
@@ -384,7 +405,7 @@ def _prepare_metadata_line(dcmdata, teil):
     return metadataline
 
 
-class DicomDirectory():
+class DicomDirectory:
     def __init__(self, dirpath, force_create_dicomdir=False, force_read=False):
         self.dicomdir_filename = 'dicomdir.pkl'
         self.standard_dicomdir_filename = 'DICOMDIR'
@@ -456,7 +477,18 @@ class DicomDirectory():
         }
         return metadata
 
-    def get_metaData(self, dcmlist, series_number, ifile=0):
+    def _get_slice_location_difference(self, dcmlist, ifile, step=1):
+        head1, teil1 = os.path.split(dcmlist[ifile])
+        head2, teil2 = os.path.split(dcmlist[ifile + step])
+
+        data1 = self._read_file(dcmlist[ifile])
+        data2 = self._read_file(dcmlist[ifile + step])
+        loc1 = get_slice_location(data1, teil1)
+        loc2 = get_slice_location(data2, teil2)
+        voxeldepth = float(np.abs(loc1 - loc2))
+        return voxeldepth
+
+    def get_metaData(self, dcmlist, series_number):
         """
         Get metadata.
         Voxel size is obtained from PixelSpacing and difference of
@@ -466,19 +498,31 @@ class DicomDirectory():
         # if dcmlist is None:
         #     dcmlist = self.files_in_serie
 
+        # number of slice where to extract metadata inforamtion
+        ifile = 0
         if len(dcmlist) == 0:
             return {}
 
         logger.debug("Filename: " + dcmlist[ifile])
         data1 = self._read_file(dcmlist[ifile])
         try:
-            head1, teil1 = os.path.split(dcmlist[ifile])
-            head2, teil2 = os.path.split(dcmlist[ifile + 1])
+            # try to get difference from the beginning and also from the end
+            voxeldepth = self._get_slice_location_difference(dcmlist, ifile)
+            voxeldepth_end = self._get_slice_location_difference(dcmlist, -2)
+            if voxeldepth != voxeldepth_end:
+                logger.warning("Depth of slices is not the same in beginning and end of the sequence")
+                voxeldepth_1 = self._get_slice_location_difference(dcmlist, 1)
+                voxeldepth = np.median([voxeldepth, voxeldepth_end, voxeldepth_1])
 
-            data2 = self._read_file(dcmlist[ifile + 1])
-            loc1 = get_slice_location(data1, teil1)
-            loc2 = get_slice_location(data2, teil2)
-            voxeldepth = float(np.abs(loc1 - loc2))
+
+
+            # head1, teil1 = os.path.split(dcmlist[ifile])
+            # head2, teil2 = os.path.split(dcmlist[ifile + 1])
+            #
+            # data2 = self._read_file(dcmlist[ifile + 1])
+            # loc1 = get_slice_location(data1, teil1)
+            # loc2 = get_slice_location(data2, teil2)
+            # voxeldepth = float(np.abs(loc1 - loc2))
         except Exception:
             logger.warning('Problem with voxel depth. Using SliceThickness')
             logger.debug(traceback.format_exc())
@@ -510,27 +554,32 @@ class DicomDirectory():
             metadata['SeriesDescription'] = data1.SeriesDescription
 
         except:
-            logger.warning(
+            logger.info(
                 'Problem with tag SeriesDescription, SeriesNumber: ' +
                 str(data1.SeriesNumber))
         try:
             metadata['ImageComments'] = data1.ImageComments
         except:
-            logger.warning(
+            logger.info(
                 'Problem with tag ImageComments')
             # , SeriesNumber: ' +
             # str(data1.SeriesNumber))
         try:
             metadata['Modality'] = data1.Modality
         except:
-            logger.warning(
+            logger.info(
                 'Problem with tag Modality')
             # SeriesNumber: ' +
             #     str(data1.SeriesNumber))
         metadata = attr_to_dict(data1, "AcquisitionDate", metadata)
         metadata = attr_to_dict(data1, "StudyDate", metadata)
+        metadata = attr_to_dict(data1, "StudyID", metadata)
         metadata = attr_to_dict(data1, "StudyDescription", metadata)
         metadata = attr_to_dict(data1, "RequestedProcedureDescription", metadata)
+        metadata = attr_to_dict(data1, "PatientSex", metadata)
+        metadata = attr_to_dict(data1, "PatientAge", metadata)
+        metadata = attr_to_dict(data1, "PatientID", metadata)
+        metadata = attr_to_dict(data1, "PatientName", metadata)
         # metadata = attr_to_dict(data1, "AcquisitionTime", metadata)
 
         metadata['dcmfilelist'] = dcmlist
@@ -545,6 +594,12 @@ class DicomDirectory():
         }
         return retval
 
+    def get_stats_of_series_in_dir_as_dataframe(self, study_id=None):
+        series_stats = self.get_stats_of_series_in_dir(study_id)
+        import pandas as pd
+        study_df = pd.DataFrame(series_stats).transpose()
+        return study_df
+
     def get_stats_of_series_in_dir(self, study_id=None):
         """ Dicom series staticstics, input is dcmdir, not dirpath
         Information is generated from dicomdir.pkl and first files of series
@@ -607,7 +662,7 @@ class DicomDirectory():
         study_info = {
 
         }
-        key = series_info.keys()[0]
+        key = list(series_info)[0]
         if "StudyDate" in series_info[key]:
             study_info["StudyDate"] = series_info[key]["StudyDate"]
         return study_info
@@ -710,7 +765,7 @@ class DicomDirectory():
         return counts, bins
 
     def get_sorted_series_files(self, startpath="", series_number=None, return_files_with_info=False,
-                                sort_keys="SliceLocation", return_files=True):
+                                sort_keys="SliceLocation", return_files=True, remove_doubled_slice_locations=True):
         """
         Function returns sorted list of dicom files. File paths are organized
         by SeriesUID, StudyUID and FrameUID
@@ -723,23 +778,27 @@ class DicomDirectory():
         :type sort_keys: One key or list of keys used for sorting method by the order of keys.
 
         """
-
-        dcmdir = sort_list_of_dicts(self.files_with_info[:], keys=sort_keys)
+        dcmdir = self.files_with_info[:]
 
         # select sublist with SeriesNumber
         if series_number is not None:
             dcmdir = [
                 line for line in dcmdir if line['SeriesNumber'] == series_number
             ]
+        dcmdir = sort_list_of_dicts(dcmdir, keys=sort_keys)
 
         logger.debug('SeriesNumber: ' + str(series_number))
 
+        if remove_doubled_slice_locations:
+            dcmdir = self._remove_doubled_slice_locations(dcmdir)
+
         filelist = []
         for onefile in dcmdir:
             filelist.append(os.path.join(startpath,
                                          self.dirpath, onefile['filename']))
             # head, tail = os.path.split(onefile['filename'])
 
+
         retval = []
         if return_files:
             retval.append(filelist)
@@ -755,6 +814,17 @@ class DicomDirectory():
 
         return retval
 
+    def _remove_doubled_slice_locations(self, dcmdir):
+        new_dcmdir = []
+        prev_slice_location = None
+        for item in dcmdir:
+            actual_slice_location = item["SliceLocation"]
+            if  actual_slice_location != prev_slice_location:
+                new_dcmdir.append(item)
+            prev_slice_location = actual_slice_location
+
+        return new_dcmdir
+
     def _create_dicomdir_info(self):
         """
         Function crates list of all files in dicom dir with all IDs
@@ -767,6 +837,9 @@ class DicomDirectory():
         for filepath in filelist:
             head, teil = os.path.split(filepath)
             dcmdata = None
+            if os.path.isdir(filepath):
+                logger.debug("Subdirectory found in series dir is ignored: " + str(filepath))
+                continue
             try:
                 dcmdata = pydicom.read_file(filepath)
 
@@ -778,9 +851,10 @@ class DicomDirectory():
                     # if e.[0].startswith("File is missing \\'DICM\\' marker. Use force=True to force reading")
                 except Exception as e:
                     if teil != self.dicomdir_filename:
-                        print('Dicom read problem with file ' + filepath)
-                    import traceback
-                    logger.debug(traceback.format_exc())
+                        # print('Dicom read problem with file ' + filepath)
+                        logger.info('Dicom read problem with file ' + filepath)
+                        import traceback
+                        logger.debug(traceback.format_exc())
             if hasattr(dcmdata, "DirectoryRecordSequence"):
                 # file is DICOMDIR - metainfo about files in directory
                 # we are not using this info
@@ -814,17 +888,23 @@ def get_slice_location(dcmdata, teil=None):
         try:
             slice_location = float(dcmdata.SliceLocation)
         except Exception as exc:
-            logger.warning("It is not possible to use SliceLocation")
-            logger.info(traceback.format_exc())
+            logger.info("It is not possible to use SliceLocation")
+            logger.debug(traceback.format_exc())
     if slice_location is None and hasattr(dcmdata, "SliceThickness") and teil is not None:
-        logger.warning(
+        logger.debug(
             "Estimating SliceLocation wiht image number and SliceThickness"
         )
 
         # from builtins import map
         i = list(map(int, re.findall('\d+', teil)))
         i = i[-1]
-        slice_location = float(i * float(dcmdata.SliceThickness))
+        try:
+            slice_location = float(i * float(dcmdata.SliceThickness))
+        except ValueError as e:
+            print(type(dcmdata.SliceThickness))
+            print(dcmdata.SliceThickness)
+            logger.debug(traceback.format_exc())
+            logger.debug("SliceThickness problem")
 
     if slice_location is None and hasattr(dcmdata, "ImagePositionPatient") and hasattr(dcmdata,
                                                                                        "ImageOrientationPatient"):
@@ -857,6 +937,35 @@ def attr_to_dict(obj, attr, dct):
         dct[attr] = getattr(obj, attr)
     return dct
 
+def get_series_number_by_guess_for_liver(dcmreader, counts, bins, qt_app=None):
+    """
+    Select the venous series from CT with around 200 images
+    :param dcmreader:
+    :param counts:
+    :param bins:
+    :param qt_app:
+    :return:
+    """
+    series_info = dcmreader.dicomdirectory.get_stats_of_series_in_dir()
+    print(dcmreader.print_series_info(series_info))
+    import pandas as pd
+    df = pd.DataFrame(list(series_info.values()))
+
+    #select CT
+    df = df[df["Modality"].str.lower().str.contains("ct") == True]
+    # select just venous
+    df = df[df["SeriesDescription"].str.lower().str.contains("ven") == True]
+    # remove saggittal
+    df = df[df["SeriesDescription"].str.lower().str.contains("sag") == False]
+    # remove cor
+    df = df[df["SeriesDescription"].str.lower().str.contains("cor") == False]
+    df["dst_to_200"] = np.abs(200 - df.Count)
+    dfs = df.sort_values(by="dst_to_200", ascending=True)
+    sn = list(dfs.SeriesNumber)[0]
+
+    return sn
+
+
 
 def get_series_number_console(dcmreader, counts, bins, qt_app=None):  # pragma: no cover
 
@@ -890,7 +999,7 @@ def get_series_number_qt(dcmreader, counts, bins, qt_app=None):  # pragma: no co
         strl = get_one_serie_info(series_info, serie_number)
         sbinsd[strl] = serie_number
         # sbins.append(str(ii) + "  " + serie_number)
-    sbins = sbinsd.keys()
+    sbins = list(sbinsd)
     snstring, ok = \
         QInputDialog.getItem(qt_app,
                              'Serie Selection',
@@ -947,7 +1056,8 @@ def sort_list_of_dicts(lst_of_dct, keys, reverse=False, **sort_args):
     # dcmdir = lst_of_dct[:]
     dcmdir = lst_of_dct
     # sorting is based on two values in tuple (has_this_key: bool, value_or_None)
-    dcmdir.sort(key=lambda x: [((False, x[key]) if key in x else (True, 0)) for key in keys], reverse=reverse, **sort_args)
+    # dcmdir.sort(key=lambda x: [((False, x[key] if x[key] is not None else 0) if key in x else (True, 0)) for key in keys], reverse=reverse, **sort_args)
+    dcmdir.sort(key=lambda x: [((False, x[key] if x[key] is not None else 0) if key in x else (True, 0)) for key in keys], reverse=reverse, **sort_args)
     # dcmdir.sort(key=lambda x: [(x[key] for key in keys], reverse=reverse, **sort_args)
     return dcmdir
 
