@@ -15,6 +15,8 @@ from io import open
 path_to_script = os.path.dirname(os.path.abspath(__file__))
 sys.path.append(os.path.join(path_to_script, "./extern/sPickle"))
 
+from .dili_subset import ndarray_to_list_in_structure
+
 def old_str_format_to_new(string):
     """
     convert old format style to new style. Works for digits only
@@ -38,15 +40,16 @@ def suggest_filename(file_path, exists=None):
     if exists:
         file_path, file_extension = os.path.splitext(file_path)
         # print(file_path)
-        m = re.search(r"\d+$", file_path)
+        m = re.search(r"_\d+$", file_path)
         if m is None:
             # cislo = 2
-            new_cislo_str = "2"
+            new_cislo_str = "_2"
         else:
             cislostr = (m.group())
-            cislo = int(cislostr) + 1
+            cislo = int(cislostr[1:]) + 1
+            # it is normal number
             file_path = file_path[:-len(cislostr)]
-            new_cislo_str = str(cislo)
+            new_cislo_str = "_" + str(cislo)
 
         file_path = file_path + new_cislo_str + file_extension  # .zfill(2)
         # trorcha rekurze
@@ -63,13 +66,16 @@ def obj_from_file(filename='annotation.yaml', filetype='auto'):
         filetype = ext[1:]
 
     if filetype in ('yaml', 'yml'):
-        import yaml
+        from ruamel.yaml import YAML
+        yaml = YAML(typ="unsafe")
         with open(filename, encoding="utf-8") as f:
-            intext = f.read()
-            obj = yaml.load(intext)
-        # f = open(filename, 'rb')
-        # obj = yaml.load(f)
-        # f.close()
+            obj = yaml.load(f)
+        if obj is None:
+            obj = {}
+        # import yaml
+        # with open(filename, encoding="utf-8") as f:
+        #     intext = f.read()
+        #     obj = yaml.load(intext)
     elif filetype in ('pickle', 'pkl', 'pklz', 'picklezip'):
         fcontent = read_pkl_and_pklz(filename)
         # import pickle
@@ -78,7 +84,10 @@ def obj_from_file(filename='annotation.yaml', filetype='auto'):
         else:
             import _pickle as pickle
         # import sPickle as pickle
-        obj = pickle.loads(fcontent)
+        if sys.version_info.major == 2:
+            obj = pickle.loads(fcontent)
+        else:
+            obj = pickle.loads(fcontent, encoding="latin1")
     else:
         logger.error('Unknown filetype ' + filetype)
     return obj
@@ -109,17 +118,23 @@ def read_pkl_and_pklz(filename):
     return fcontent
 
 
-def obj_to_file(obj, filename, filetype='auto'):
+def obj_to_file(obj, filename, filetype='auto', ndarray_to_list=False, squeeze=True):
     '''Writes annotation in file.
 
-    Filetypes:
+    :param filetype:
+        auto
         yaml
         pkl, pickle
         pklz, picklezip
+    :param ndarray_to_list: convert ndarrays in obj to lists
+    :param squeeze: squeeze ndarray
+
     '''
     # import json
     # with open(filename, mode='w') as f:
     #    json.dump(annotation,f)
+    if ndarray_to_list:
+        obj = ndarray_to_list_in_structure(obj, squeeze=squeeze)
 
     # write to yaml
     d = os.path.dirname(os.path.abspath(filename))
@@ -131,13 +146,17 @@ def obj_to_file(obj, filename, filetype='auto'):
         filetype = ext[1:]
 
     if filetype in ('yaml', 'yml'):
-        import yaml
-        if sys.version_info.major == 2:
-            with open(filename, 'wb') as f:
-                yaml.dump(obj, f, encoding="utf-8")
-        else:
-            with open(filename, "w", encoding="utf-8") as f:
-                yaml.dump(obj, f)
+        # import yaml
+        from ruamel.yaml import YAML
+        yaml = YAML(typ="unsafe")
+        with open(filename, 'wt', encoding="utf-8") as f:
+            yaml.dump(obj, f)
+        # if sys.version_info.major == 2:
+        #     with open(filename, 'wb') as f:
+        #         yaml.dump(obj, f, encoding="utf-8")
+        # else:
+        #     with open(filename, "w", encoding="utf-8") as f:
+        #         yaml.dump(obj, f)
     elif filetype in ('pickle', 'pkl'):
         f = open(filename, 'wb')
         logger.info("filename " + filename)
@@ -246,12 +265,11 @@ def resize_to_mm(data3d, voxelsize_mm, new_voxelsize_mm, mode='nearest'):
     import scipy
     import scipy.ndimage
 
-    if new_voxelsize_mm == 'orig':
+    if np.all(list(new_voxelsize_mm) == 'orig'):
         new_voxelsize_mm = np.array(voxelsize_mm)
-
-    elif new_voxelsize_mm == 'orig*2':
+    elif np.all(list(new_voxelsize_mm) == 'orig*2'):
         new_voxelsize_mm = np.array(voxelsize_mm) * 2
-    elif new_voxelsize_mm == 'orig*4':
+    elif np.all(list(new_voxelsize_mm) == 'orig*4'):
         new_voxelsize_mm = np.array(voxelsize_mm) * 4
         # vx_size = np.array(metadata['voxelsize_mm']) * 4
 
@@ -264,21 +282,51 @@ def resize_to_mm(data3d, voxelsize_mm, new_voxelsize_mm, mode='nearest'):
     ).astype(data3d.dtype)
     return data3d_res
 
-def use_economic_dtype(data3d):
+def suits_with_dtype(mn, mx, dtype):
+    """
+    Check whether range of values can be stored into defined data type.
+    :param mn: range minimum
+    :param mx: range maximum
+    :param dtype:
+    :return:
+    """
+    type_info = np.iinfo(dtype)
+    if mx <= type_info.max and mn >= type_info.min:
+        return True
+    else:
+        return False
+
+def use_economic_dtype(data3d, slope=1, inter=0, dtype=None):
     """ Use more economic integer-like dtype if it is possible.
 
     :param data3d:
+    :param dtype: if dtype is not used, the automatic is used
     :return:
     """
-    dtype = data3d.dtype
-    if issubclass(dtype.type, np.integer):
 
-        i16 = np.iinfo(np.int16)
-        # i8 = np.iinfo(np.int8)
-        mx = data3d.max()
-        mn = data3d.min()
-        if mx < i16.max and mn > i16.min:
-            data3d = data3d.astype(np.int16)
+    if dtype is None:
+        dtype = data3d.dtype
+        if issubclass(dtype.type, np.integer):
+            mn = data3d.min() * slope + inter
+            mx = data3d.max() * slope + inter
+            if suits_with_dtype(mn, mx, dtype=np.uint8):
+                dtype = np.uint8
+            elif suits_with_dtype(mn, mx, dtype=np.int8):
+                dtype = np.int8
+            elif suits_with_dtype(mn, mx, dtype=np.uint16):
+                dtype = np.uint16
+            elif suits_with_dtype(mn, mx, dtype=np.int16):
+                dtype = np.int16
+            elif suits_with_dtype(mn, mx, dtype=np.uint32):
+                dtype = np.uint32
+            elif suits_with_dtype(mn, mx, dtype=np.int32):
+                dtype = np.int32
 
-    return data3d
+    # new_data3d = ((np.float(slope) * data3d) + np.float(inter)).astype(dtype)
+    if slope == 1 and inter == 0:
+        # this can prevent out of memmory
+        new_data3d = data3d.astype(dtype)
+    else:
+        new_data3d = ((slope * data3d) + inter).astype(dtype)
+    return new_data3d
 
