@@ -28,14 +28,16 @@ try:
     dicom.debug(False)
 except:
     import pydicom as dicom
-
+    dicom.config.debug(False)
 #
 import io3d
+import io3d.datasets
 import io3d.dcmreaddata as dcmr
-sample_data_path = "~/data/medical/orig/sample_data/"
-sample_data_path = op.expanduser(sample_data_path)
-
+# sample_data_path = "~/data/medical/orig/sample_data/"
+# sample_data_path = op.expanduser(sample_data_path)
+sample_data_path = io3d.datasets.join_path("sample_data")
 
+alternative_data_path = "e:\\data\\medical\\orig\\"
 
 class DicomReaderTest(unittest.TestCase):
     interactivetTest = False
@@ -51,8 +53,8 @@ class DicomReaderTest(unittest.TestCase):
 # voxelsize depth = 5 mm
         self.assertEqual(metadata['voxelsize_mm'][0], 5)
 
-    def test_dicomreadircad(self):
-        dcmdir = io3d.datasets.join_path("3Dircadb1.1/PATIENT_DICOM")
+    def test_dicomread_ircad(self):
+        dcmdir = io3d.datasets.join_path("3Dircadb1.1", "PATIENT_DICOM")
         # dcmdir = '/home/mjirik/data/medical/data_orig/jatra-kma/jatra_5mm/'
         # self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
         data3d, metadata = io3d.datareader.read(dcmdir, dataplus_format=False)
@@ -68,13 +70,30 @@ class DicomReaderTest(unittest.TestCase):
         self.assertEqual(data3d[0, 0, 0], -1024)
         self.assertEqual(np.max(data3d), 1023)
 
+    def test_dicomread_jatra5mm(self):
+        dcmdir = io3d.datasets.join_path("jatra_5mm")
+        # dcmdir = '/home/mjirik/data/medical/data_orig/jatra-kma/jatra_5mm/'
+        # self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
+        data3d, metadata = io3d.datareader.read(dcmdir, dataplus_format=False)
+        #slice size is 512x512
+        self.assertEqual(data3d.shape[0], 93)
+        self.assertEqual(data3d.shape[2], 512)
+
+        self.assertEqual(np.round(metadata['voxelsize_mm'][0], 2), 5.0)
+        self.assertEqual(np.round(metadata['voxelsize_mm'][1], 2), 0.75)
+        self.assertEqual(np.round(metadata['voxelsize_mm'][2], 2), 0.75)
+
+        # intensity check
+        self.assertEqual(data3d[0, 0, 0], -1024)
+        self.assertEqual(np.max(data3d), 1503)
 
     def test_dicomread_read_corrupted_dcmdir_file(self):
         dcmdir = os.path.join(sample_data_path, '../sample_data/jatra_5mm')
-        pth_dicomdir = os.path.join(
-                sample_data_path, '../sample_data/jatra_5mm/dicomdir.pkl')
-        pth_dicomdir_bck = os.path.join(
-                sample_data_path, '../sample_data/jatra_5mm/dicomdir.pkl.bck')
+        dcmdir = io3d.datasets.join_path("sample_data", "jatra_5mm")
+        pth_dicomdir = io3d.datasets.join_path(
+                 '../sample_data/jatra_5mm/dicomdir.pkl')
+        pth_dicomdir_bck = io3d.datasets.join_path(
+                '../sample_data/jatra_5mm/dicomdir.pkl.bck')
         import shutil
 # Backup file
         shutil.copy2(pth_dicomdir, pth_dicomdir_bck)
@@ -145,8 +164,8 @@ class DicomReaderTest(unittest.TestCase):
         # plt.imshow(overlay[1][:,:,0])
         # plt.show()
 
-        self. assertEqual(overlay[1][0, 200, 200], 1)
-        self. assertEqual(overlay[1][0, 100, 100], 0)
+        self.assertEqual(overlay[1][0, 200, 200], 1)
+        self.assertEqual(overlay[1][0, 100, 100], 0)
 
     def test_read_volumetry_overlay_with_dicom_module(self):
         """
@@ -244,6 +263,34 @@ class DicomReaderTest(unittest.TestCase):
         self.assertTrue(info_str,
                         '7 (93, CT, DE_Abdom_1F  5.0  B30f M_0.3, )\n')
 
+
+    @attr('dataset')
+    def test_dcmread_perfusion_data_with_strange_serieses(self):
+        # there was problem with DICOMDIR file
+
+        dcmdir = alternative_data_path + "perfusion\\32584640\\"
+        # os.path.join(sample_data_path, '../sample_data/jatra_5mm')
+        # dcmdir = '/home/mjirik/data/medical/data_orig/jatra-kma/jatra_5mm/'
+        # self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
+        reader = dcmr.DicomReader(dcmdir, force_create_dicomdir=True, series_number=17)
+        data3d = reader.get_3Ddata()
+        metadata = reader.get_metaData()
+        stats = reader.dcmdirstats()
+        info_str = reader.print_series_info(stats, minimal_series_number=0)
+
+        # import sed3
+        # ed = sed3.sed3(data3d)
+        # ed.show()
+        # slice size is 512x512
+        self.assertEqual(data3d.shape[2], 512)
+        # voxelsize depth = 5 mm
+        self.assertLess(data3d[0, 0, 0], -1000)
+        # in the middle of image should be density higher than 50
+        self.assertGreater(np.max(data3d[:, 100:-100, 100:-100]), 50)
+
+        # self.assertTrue(info_str,
+        #                 '7 (93, CT, DE_Abdom_1F  5.0  B30f M_0.3, )\n')
+
     @attr('dataset')
     def test_dcmread_some_strange_data(self):
         # there was problem with DICOMDIR file
@@ -298,7 +345,23 @@ class DicomReaderTest(unittest.TestCase):
         self.assertEqual(data3d.shape[2], 512)
         self.assertEqual(metadata['voxelsize_mm'][0], 5)
 
+    @attr('dataset')
+    def test_dcmread_guess_series_number(self):
+
+        dcmdir = os.path.join(sample_data_path, '../sample_data/jatra_5mm')
+        dcmdir = os.path.join(sample_data_path, '../erazmusplus/44204675/')
+        # dcmdir = '~/data/medical/orig/'
+        # dcmdir = '/home/mjirik/data/medical/data_orig/jatra-kma/jatra_5mm/'
+        # self.data3d, self.metadata = dcmr.dcm_read_from_dir(self.dcmdir)
+        # spravne cislo serie je 7
+        reader = dcmr.DicomReader(dcmdir, series_number=None, get_series_number_callback="guess for liver")
+        data3d = reader.get_3Ddata()
+        metadata = reader.get_metaData()
+        self.assertEqual(data3d.shape[2], 512)
+        self.assertEqual(metadata['SeriesNumber'], 11)
+        # self.assertEqual(metadata['voxelsize_mm'][0], 5)
     # @unittest.skipIf(not interactivetTest, 'interactiveTest')
+
     @attr('dataset')
     def test_dcmread_select_series_reasmusplus(self):
 
@@ -398,6 +461,56 @@ class DicomReaderTest(unittest.TestCase):
         print(sorted_files_with_info)
         self.assertTrue(op.exists(sorted_files[0]))
 
+    @attr('dataset')
+    def test_read_big_big_data(self):
+        """
+        files in nejlepsi_rozli_nevycistene
+        """
+        dcmdir = r"E:\data\medical\orig\ct_porcine_liver\P01\29-8-12-a\Nejlep_rozli_nevycistene"
+        io3d.read(dcmdir)
+        # print(sorted_files_with_info)
+        # self.assertTrue(op.exists(sorted_files[0]))
+
+    @attr('dataset')
+    def test_read_dataset_where_is_voxelsize_equal_zero(self):
+        """
+        voxelsize_mm[0] should be grater than zero
+        """
+        dcmdir = r"E:\data\medical\orig\ct_porcine_liver\P03\23-9-13_23913\Makro\VEN_ABDOMEN_5_0_B31S_0002"
+        # dcmdir = r"E:\data\medical\orig\ct porctine liver\P01\29-8-12-a\Nejlep_rozli_nevycistene"
+        datap = io3d.read(dcmdir)
+
+        self.assertGreater(datap["voxelsize_mm"][0], 0)
+        # print(sorted_files_with_info)
+        # self.assertTrue(op.exists(sorted_files[0]))
+
+    @attr('dataset')
+    def test_read_and_write_dataset_to_fix_problem_with_out_of_memory(self):
+        """
+        voxelsize_mm[0] should be grater than zero
+        """
+        dcmdir = r"E:\data\medical\orig\ct_porcine_liver\P01\29-8-12-a\MIkroCT-nejhrubsi_rozliseni\DICOM_liver-1st-important_Macro_pixel-size53.0585um"
+        # dcmdir = r"E:\tmp\data\medical\orig\ct_porcine_liver\P01\29-8-12-a\MIkroCT-nejhrubší rozlišení\DICOM_liver-1st-important_Macro_pixel-size53.0585um"
+        # dcmdir = r"E:\data\medical\orig\ct porctine liver\P01\29-8-12-a\Nejlep_rozli_nevycistene"
+        datap = io3d.read(dcmdir)
+
+        io3d.write(datap, "~/tmp/nejhrubsi.mhd")
+        self.assertGreater(datap["voxelsize_mm"][0], 0)
+
+    def test_save_big_data(self):
+        output_filename = "~/tmp/big_data.mhd"
+        if op.exists(op.expanduser(output_filename)):
+            os.remove(op.expanduser(output_filename))
+        data3d = np.zeros([1024, 1024, 1024], dtype=np.uint16)
+        voxelsize_mm = [0.5, 0.5, 0.5]
+        datap = dict(data3d=data3d, voxelsize_mm=voxelsize_mm)
+        io3d.write(datap, output_filename)
+
+        self.assertTrue(op.exists(op.expanduser(output_filename)))
+        # clean
+        if op.exists(op.expanduser(output_filename)):
+            os.remove(op.expanduser(output_filename))
+
     def test_is_dicomdir_information_about_files(self):
         """
         files in vincentka_sample have no extension
@@ -408,9 +521,20 @@ class DicomReaderTest(unittest.TestCase):
 
         sorted_files, sorted_files_with_info = dicomdirectory.get_sorted_series_files(sort_keys=["AcquisitionTime", "SeriesNumber"], return_files_with_info=True)
         # self.assertEqual(metadata["voxelsize_mm"][1], 512)
-        print(sorted_files_with_info)
+        # print(sorted_files_with_info)
         self.assertTrue(op.exists(sorted_files[0]))
 
+
+    def test_parts_of_pyqt_get_series_number(self):
+        dcmdir = op.join(sample_data_path, 'vincentka_sample/')
+        dicomdirectory = dcmr.DicomDirectory(dcmdir)
+        files_with_info = dicomdirectory.files_with_info
+
+        series_info = dicomdirectory.get_stats_of_series_in_dir()
+        message = dicomdirectory.get_study_info_msg(series_info=series_info)
+
+        self.assertTrue(type(message) is str)
+
     # @attr('actual')
     def test_is_dicomdir(self):
         """
@@ -449,7 +573,6 @@ class DicomReaderTest(unittest.TestCase):
         # ed = sed3.sed3(data3d)
         # ed.show()
 
-    @attr('actual')
     def test_compare_dcmread_and_dataread(self):
 
         # dcmdir = os.path.join(path_to_script, '../vincentka_2013_06mm/')
